<?php

/*
 * Copyright 2009 Andrés Javier López
 *
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * Maneja un arreglo de buffers de salida
 *
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */

class Buffer extends Singleton
{	
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************

	/**
	 * Instancia única de la clase
	 * @var Buffer
	 */
	
	private static $instance;
	
	/**
	 * Arreglo que almacena los buffers de salida
	 * @var array
	 */
	private $buffer = array();

	/**
	 * Pila que almacena los identificadores de los buffers
	 * @var array
	 */
	private $stack = array();

	//**********************************************************************************
	// Métodos mágicos
	//**********************************************************************************

	/**
	 * Constructor privado para evitar otras instancias
	 */

	private function __construct()
	{
	}

	//**********************************************************************************
	// Métodos para el procesamiento de acciones	
	//**********************************************************************************

	/**
	 * Activa un buffer de salida con el indentificador asignado
	 * @param string $buffer_id
	 * @return void
	 */
	public function start( $buffer_id )
	{
		$this->stack[] = $buffer_id;

		ob_start();
	}

	/**
	 * Finaliza el buffering de la salida y almacena su contenido
	 * @return void
	 */
	public function end()
	{
		$buffer_id = array_pop( $this->stack );

		$contents = ob_get_contents();

		if( $contents && $buffer_id )
		{
			if( isset( $this->buffer[ $buffer_id ] ))
			{
				$this->buffer[ $buffer_id ] .= $contents;
			}
			else
			{
				$this->buffer[ $buffer_id ] = $contents;
			}
			
			ob_end_clean();
		}
	}
	
	/**
	 * Devuelve el contenido del buffer de salida que se encuentra almacenado con el id especificado
	 * @param string $buffer_id
	 * @param boolean $clean Limpia el contenido del buffer, por defecto es verdadera
	 * @return string Si el buffer no existe devuelve una cadena vacía
	 */
	public function getContent( $buffer_id, $clean = true )
	{
		if( !isset( $this->buffer[$buffer_id] ) )
		{
			return '';
		}
		
		$buffer = $this->buffer[$buffer_id];
		
		if( $clean )
		{
			$this->buffer[$buffer_id] = '';
		}
		
		return $buffer;		
	}
	
	/**
	 * Limpia el contenido almacenado del buffer con el id especificado
	 * @param string $buffer_id
	 * @return void
	 */
	public function clean( $buffer_id )
	{
		$this->buffer[$buffer_id] = '';
	}
	
	/**
	 * Escribe el contenido directamente en el buffer almacenado
	 * @param string $buffer_id
	 * @param string $content
	 * @param boolean $overwrite Si es verdadero sobreescribe el contenido anterior. Por defecto es falso.
	 * @return void
	 */
	public function write( $buffer_id, $content, $overwrite = false )
	{
		if( !isset($this->buffer[$buffer_id]) || $overwrite )
		{
			$this->buffer[$buffer_id] = $content;
		}
		else
		{
			$this->buffer[$buffer_id] .= $content;
		}
	}
	
	//**********************************************************************************
	// Métodos estáticos	
	//**********************************************************************************
	
	/**
	 * Devuelve la instancia única de la clase
	 * @return Buffer
	 */

	public static function getInstance()
	{
		if( !self::$instance instanceof self )
		{
			self::$instance = new self;
		}

		return self::$instance;
	}
}

?>
