<?php

/*
 * Copyright 2009 Andrés Javier López
 *
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * Controlador base para MoonDragon
 *
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 * @abstract
 */

abstract class Controller
{
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************

	/**
	 * Tarea por defecto
	 * @var string
	 */
	protected $default_task;

	/**
	 * Modelo por defecto
	 * @var string
	 */
	protected $default_model;

	/**
	 * Vista por defecto
	 * @var string
	 */
	protected $default_view;

	//**********************************************************************************
	// Métodos mágicos
	//**********************************************************************************

	/**
	 * Constructor base del controlador
	 * @param string $default_task
	 * @param string $default_model
	 * @param string $default_view
	 * @return void
	 * @final
	 */
	
	final public function __construct( $default_task = '', $default_model = '', $default_view = '' )
	{
		$this->default_task = ( $default_task == '' )?'display':$default_task;
		$this->default_model = $default_model;
		$this->default_view = $default_view;
		
		$this->init();
		
		Registry::set( 'controller', $this );
	}

	/**
	 * Procesa las llamadas a las tareas desconocidas
	 * @param string $name
	 * @return void
	 */
	
	public function __call( $name, $params )
	{
		trigger_error( 'La tarea '.$name.' no existe para el controlador '.get_class($this), E_USER_WARNING );
		$this->error();
	}

	//**********************************************************************************
	// Métodos para el manejo de variables
	//**********************************************************************************

	/**
	 * Modifica la tarea por defecto del controlador
	 * @param string $task
	 * @return void
	 */
	
	public function setDefaultTask( $task )
	{
		$this->default_task = ($task == '' )?'display':$task;
	}

	/**
	 * Modifica el modelo por defecto del controlador
	 * @param string $model
	 * @return void
	 */
	public function setDefaultModel( $model )
	{
		$this->default_model = $model;
	}

	/**
	 * Modifica la vista por defecto del controlador
	 * @param string $view
	 * @return void
	 */
	public function setDefaultView( $view )
	{
		$this->default_view = $view;
	}

	/**
	 * Devuelve el modelo solicitado o el modelo por defecto
	 * @param string[optional] $model
	 * @return Model
	 */
	
	public function getModel( $model = '' )
	{
		if( $model == '' )
		{
			$model = $this->default_model;	
		}

		if( !( class_exists( $model ) && is_subclass_of( $model, 'Model' ) ) )
		{
			trigger_error( 'No existe el modelo '.$model , E_USER_NOTICE );
			$model = Registry::get( 'DEFAULT_MODEL' );
		}
		
		return new $model();
	}
	
	/**
	 * Devuelve la vista por defecto o la vista solicitada
	 * @param string[optional] $view
	 * @return View
	 */
	public function getView( $view = '' )
	{
		if( $view == '' )
		{
			$view = $this->default_view;	
		}

		if( !( class_exists( $view ) && is_subclass_of( $view, 'View' ) ) )
		{
			trigger_error( 'No existe la vista '.$view , E_USER_NOTICE );
			$view = Registry::get( 'DEFAULT_VIEW' );
		}
		
		return new $view();
	}

	//**********************************************************************************
	// Métodos para el procesamiento de acciones	
	//**********************************************************************************

	/**
	 * Ejecuta la tarea por defecto o la tarea seleccionada
	 * @param string[optional] $task
	 * @return void
	 */
	
	public function run( $task = '' )
	{
		if( $task == '' )
		{
			$default_task = $this->default_task;
			$this->$default_task();
		}
		else
		{
			$this->$task();
		}
	}

	//**********************************************************************************
	// Métodos privados	
	//**********************************************************************************

	/**
	 * Tarea por defecto
	 * @abstract
	 * @return void
	 */
	abstract protected function display();

	/**
	 * Procesamiento de errores por defecto
	 * @abstract
	 * @return void
	 */
	abstract protected function error();
	
	/**
	 * Procesamiento de secciones no autorizadas
	 * @return void
	 */
	
	protected function unauthorized()
	{
		// Agrega variable global con mensaje de sección no autorizada
		Vars::add( 'Error_Message', 'No esta autorizado a ver esta sección' ); // @todo localizar contenido
		
		$this->error();
	}
	
	/**
	 * Procesamiento de secciones no existentes
	 * @return void
	 */
	
	protected function unexistent()
	{
		// Agrega variable global con mensaje de sección inexistente
		Vars::add( 'Error_Message', 'La sección seleccionada no existe' ); // @todo localizar contenido
		
		$this->error();
	}
	
	/**
	 * Publica un mensaje de error
	 * 
	 * @param string $string
	 * @return void
	 */
	
	protected function setError( $string )
	{
		Vars::add( 'Error_Message', $string );
		
		$this->error();
	}
	
	/**
	 * Inicializa las propiedades del objeto
	 * @return void
	 */
	protected function init()
	{
	}
	
}

?>
