<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Gestor de errores personalizado del sistema
 * 
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */
 
class ErrorHandler extends Singleton
{
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************
	
	/**
	 * Instancia única de la clase
	 * @var ErrorHandler
	 */
	
	private static $instance;
	
	/**
	 * Controla la salida de mensajes de depuración
	 * @var boolean
	 */
	private $debug = true;
	
	/**
	 * Controla la salida de mensajes de depuracion internos de php
	 * @var boolean
	 */
	private $system_info = false;
	
	/**
	 * Controla la salida de mensajes de advertencia sobre fuciones deprecadas
	 * @var boolean
	 */
	private $deprecated = true;
	
	/**
	 * Nombre de la función utilizada para detener la ejecución del sistema
	 * @var callback
	 */
	private $halt_callback = '';
		
	//**********************************************************************************
	// Métodos mágicos
	//**********************************************************************************
	
	/**
	 * Constructor privado de la clase
	 * Registra el nuevo manejador de errores
	 * @return void
	 */
	private function __construct()
	{
		set_error_handler( array( $this, 'handler' ) );
	}
		
	//**********************************************************************************
	// Métodos para el manejo de variables
	//**********************************************************************************
	
   	/**
   	 * Modifica los parámetros de activación de los mensajes de depuración del sistema
   	 * @param boolean[opcional] $debug
   	 * @param boolean[opcional] $system_info
   	 * @param boolean[opcional] $deprecated
   	 * @return void
   	 */
	public function setDebugMode( $debug = true, $system_info = false, $deprecated = true )
	{
		$this->debug = $debug;
		$this->system_info = $system_info;
		$this->deprecated = $deprecated;
	}
	
	/**
	 * Activa o desactiva los mensajes de depuración
	 * @param boolean[opcional] $debug
	 * @return void
	 */
	public function setDebug( $debug = true )
	{
		$this->debug = $debug;
	}
	
	/**
	 * Activa o desactiva los mensajes de depuración internos de php
	 * @param boolean[opcional] $system_info
	 * @return void
	 */
	public function setSystemInfo( $system_info = false )
	{
		$this->system_info = $system_info;
	}
	
	/**
	 * Activa o desactiva los mensajes de advertencia sobre uso de funciones deprecadas
	 * @param boolean[opcional] $deprecated
	 * @return void
	 */
	public function setDeprecated( $deprecated = true )
	{
		$this->deprecated = $deprecated;
	}
	
	/**
	 * Modifica la función utilizada para detener la ejecución del sistema
	 * @param callback $function_name funcion( $error_string, $error_file, $error_line ) 
	 * @return void
	 */	
	public function setHaltHandler( $function_name )
	{
		$this->halt_callback = $function_name;
	}
	
	//**********************************************************************************
	// Métodos para el procesamiento de acciones	
	//**********************************************************************************
	
	/**
	 * Manejador de error personalizado
	 * @param int $error_number
	 * @param string $error_string
	 * @param string $error_file
	 * @param int $error_line
	 * @param array $error_context
	 * @return boolean
	 */
	public function handler( $error_number, $error_string, $error_file, $error_line, $error_context )
	{
		//Si se ha desactivado la generación de errores se sale de la función
		if( error_reporting() == 0 )
		{
			return true;
		}
		
		switch( $error_number )
		{
			case E_USER_ERROR:
				
				$error_log = date( 'd/m/Y H:i:s' )."\n";// @todo localizar contenido
				$error_log .= 'Error Fatal: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n";// @todo localizar contenido
				$error_log .= "Contexto:\n".print_r( $error_context, true )."\n\n";// @todo localizar contenido
				
				file_put_contents( 'logs/error.log', $error_log, FILE_APPEND | LOCK_EX );
				
				$halt_callback = $this->halt_callback;
				if( function_exists( $halt_callback ) )
				{
					$halt_callback( $error_string, $error_file, $error_line );
				}
				else
				{
					die( 'Error Fatal: "'.$error_string.'" en '.$error_file.' línea '.$error_line.'. Consulte el log de errores para obtener más detalles.' ); //@todo Al implementar soporte para idiomas múltiples se reemplazará el texto por un soporte localizado
				}
				break;
			case E_WARNING:
			case E_USER_WARNING:
				
				$buffer = Buffer::getInstance();
				
				$buffer->write( "e_warning", 'Advertencia: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n" );// @todo localizar contenido
				
				break;
			case E_USER_NOTICE:
				if( !$this->debug )
				{
					return true;
				}
				
				$buffer = Buffer::getInstance();
				
				$buffer->write( "e_notice", 'Mensaje: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n" );// @todo localizar contenido
								
				break;
			case E_NOTICE:
			case E_STRICT:
				if( !$this->system_info )
				{
					return true;
				}
				
				$buffer = Buffer::getInstance();
				
				$buffer->write( "mensajes", 'Mensaje: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n" );// @todo localizar contenido
								
				break;
			case E_DEPRECATED:
			case E_USER_DEPRECATED:
				if( !$this->deprecated )
				{
					return true;
				}
				
				$buffer = Buffer::getInstance();
				
				$buffer->write( "mensajes", 'Mensaje: "'.$error_string.'" en '.$error_file.' línea '.$error_line."\n" );// @todo Localizar contenido
								
				break;
			default:
				//error desconocido, se pasa el control al gestor por defecto de php
				return false;
		}
				
		return true;
	}
	
	//**********************************************************************************
	// Métodos estáticos	
	//**********************************************************************************
	
	/**
	 * Devuelve la instancia única de la clase
	 * @return ErrorHandler
	 */

	public static function getInstance()
	{
		if( !self::$instance instanceof self )
		{
			self::$instance = new self;
		}

		return self::$instance;
	}
	
	/**
	 * Inicializa el gestor de errores
	 * @param boolean[optional] $error
	 * @param boolean[optional] $system_info
	 * @param boolean[optional] $deprecated
	 * @return void
	 */
	
	public static function init( $error = true, $system_info = false, $deprecated = true )
	{
		$md_error = ErrorHandler::getInstance();
		$md_error->setDebug( $error );
		$md_error->setSystemInfo( $system_info );
		$md_error->setDeprecated( $deprecated );
	}
}
 
 ?>
 