<?php

/*
 * Copyright 2009 Andrés Javier López
 * 
 * This file is part of MoonDragon.
 *
 * MoonDragon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * MoonDragon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MoonDragon.  If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

/**
 * Se encarga de cargar las librerias
 * 
 * @todo La carga automática de librerías puede ser de utilidad, pero no la necesitamos por el momento
 * @author Andrés Javier López <ajavier.lopez@gmail.com>
 * @copyright Andrés Javier López 2009
 */
 
 class Libs
{	
	//**********************************************************************************
	// Variables Privadas
	//**********************************************************************************
	
	/**
	 * Listado de librerías
	 * @var array
	 */
	private static $libraries;

	//**********************************************************************************
	// Métodos privados	
	//**********************************************************************************
	
	/**
	 * Comprueba si la librería es compatible con el sistema
	 * 
	 * @param Lib $libobject
	 * @return boolean
	 */
	private static function checkCompatibility( Lib $libobject )
	{
		$md_version = self::getVersion( Registry::get( 'MOONDRAGON_VERSION' ) );
		$lib_version = self::getVersion( $libobject->moondragon_version );
		
		return self::checkVersion( $md_version, $lib_version );
	}
	
	/**
	 * Comprueba si las dependencias de la librería se satisfacen
	 * 
	 * @param Lib $libobject
	 * @return boolean
	 */
	private static function checkDependencies( Lib $libobject )
	{
		$dependencies = $libobject->getDependencies();
		
		foreach( $dependencies as $lib => $version )
		{
			if( !isset( self::$libraries[$lib] ) )
			{
				if( !self::import( $lib  ) )
				{
					return false;
				}
			}
			
			if( !self::checkVersion( self::getVersion( self::$libraries[$lib]->version ), self::getVersion( $version ) ) )
			{
				trigger_error( 'La versión de '.$lib.' es incompatible', E_USER_WARNING ); // @todo localización pendiente
				return false;	
			}
		}
		
		return true;
	}
	
	/**
	 * Desglosa los números de versión en un arreglo a partir de una cadena
	 * 
	 * @param string $version
	 * @return array
	 */
	
	private static function getVersion( $version )
	{
		list( $ver, $devel ) = explode( ' ', $version );
		
		list( $mayor, $minor, $bugfixes ) = explode( '.', $ver );
		
		if( $devel != '' )
		{
			list( $state, $number ) = explode( '_', $devel );
		}
		else
		{
			$state = 'stable';
			$number = '';
		}
		
		return array( 'mayor' => $mayor, 'minor' => $minor, 'bugfixes' => $bugfixes, 'state' => $state, 'number' => $number );
	}
	
	/**
	 * Comprueba la compatibilidad entre dos versiones
	 * 
	 * @param array $actual_version
	 * @param array $required_version
	 * @return boolean
	 */
	
	private static function checkVersion( $actual_version, $required_version )
	{
		if( $actual_version['mayor'] == $required_version['mayor'] )
		{
			if( $actual_version['minor'] == 'x' || $actual_version['minor'] >= $required_version['minor'] )
			{
				if( $actual_version['state'] == 'stable' )
				{
					return true;
				}
				else
				{
					if( ( $actual_version['state'] == 'alpha' && $required_version['state'] == 'alpha' ) || ( $actual_version == 'beta' && ( $required_version == 'alpha' || $required_version == 'beta') ) )
					{
						if( $actual_version['number'] >= $required_version['number'] )
						{
							return true;
						}
					} 
				}
			}
		}
		
		return false;
	}
	
	//**********************************************************************************
	// Métodos estáticos	
	//**********************************************************************************
	
	/**
	 * Agrega una librería al sistema
	 * 
	 * @param string $library
	 * @return boolean
	 */
	public static function import( $library )
	{
		if( isset( self::$libraries[$library] ) )
		{
			return true;
		}
		
		if( (include_once 'include/lib/'.strtolower($library).'/lib.'.strtolower($library).'.inc') != 1 )
		{
			trigger_error( 'No se ha podido incluir la librería '.$library, E_USER_ERROR ); // @todo localización pendiente
			return false;
		}
		
		$libclass = $library.'Lib';
		
		if( class_exists( $libclass ) && is_subclass_of( $libclass, 'Lib' ) )
		{
			$libobject = new $libclass();
			
			if( self::checkCompatibility( $libobject ) && self::checkDependencies( $libobject ) )
			{
				self::$libraries[ $library ] = $libobject;
			}
			else
			{
				trigger_error( 'La librería '.$library.' no es compatible con esta instalación de MoonDragon', E_USER_ERROR ); // @todo localización pendiente
				return false;
			}			
		}
		else
		{
			trigger_error( 'No existe un objeto para la librería '.$library, E_USER_WARNING ); // @todo localización pendiente
		}
		
		return true;
	}
	
	/**
	 * Ejecuta los procesos automáticos de las librerías importadas
	 * 
	 * @return void
	 */
	public static function autorun()
	{
		if( !empty( self::$libraries ) )
		{
			foreach( self::$libraries as $libobject )
			{
				$libobject->autorun();
			}
		}
	}
}
 
?>
 